%
% This script demonstrates the process of ARMA modeling and prediction
% for the chemical batch process data.  
%
% First, clear memory and reset the RNG's.  
%
clear
rand('state',0);
randn('state',0);
%
% load in the data.
%
load batch
%
% Take out the mean.
%
batchmean=mean(batch)
batch=batch-batchmean;
%
% Estimate the autocorrelations.
%
myr=zeros(20,1);
for k=1:20
  myr(k)=r(batch,k);
end
figure(1);
plot(0:20,[1; myr],'ko');
xlabel('lag k');
ylabel('r_k');
print -deps batchr.eps
%
% It appears that the autocorrelations follow and exponentially
% damped sine wave, but after k=5 or k=6, this dies down to noise
% level.  
%
%
% Estimate the PACF.
%
phihat=pacf(myr,5)
%
% It appears that phihat(1,1) and phihat(2,2) are nonzero, but
% after that, the values of phihat are quite small and could just
% be due to random sampling variability.  
%
% From the PACF and autocorrelations, it appears that an ARMA(2,0) model
% is appropriate for these data.  Estimate phi1 and phi2.  
%
phi1=(myr(1)*(myr(2)-1))/(myr(1)^2-1)
phi2=(myr(1)^2-myr(2))/(myr(1)^2-1)
%
% We get phi1=-0.3198, phi2=0.1797. 
%
% So, our model is z(n)=a(n)-0.3198*z(n-1)+0.1797*z(n-2).  The phi
% function is phi(B)=1+0.3198*B-0.1979*B^2.  The zeros of phi(B)
% are at B=-1.6 and B=3.4.  Since these zeros are outside of the unit
% circle, we have a stationary ARMA process.  
%
% 
% Next, we estimate the a's.  
%
%
% We don't know anything before time 1, so we treat all earlier z's
% as if they were 0.  The transient effect of this goes away as
% time goes on.
%
% Since z(1)=a(1)-0.3198*z(0)+0.1797*z(-1), we estimate a(1)=z(1).
% Since z(2)=a(2)-0.3198*z(1)+0.1797*z(0), we estimate a(2)=z(2)+0.3198*z(1).
%
a=zeros(size(batch));
a(1)=batch(1);
a(2)=batch(2)-phi1*batch(1);
%
% From here on in, we can use previous z's to predict the next z.
% The error in this prediction is our estimate of a.  
%
%   zpred(n)=-0.3198*z(n-1)+0.1797*z(n-2)
%   z(n)=a(n)-0.3198*z(n-1)+0.1797*z(n-2)
%
% So, we get
%
%   a(n)=z(n)-zpred(n)
%
for k=3:length(batch)
 batchnext=phi1*batch(k-1)+phi2*batch(k-2);
 a(k)=batch(k)-batchnext;
end
%
% Estimate sigmaa. 
%
sigmaa=std(a)
%
% Now, predict the future z's.  The prediction formula is 
%
%  z(k+1)=phi(1)*z(k)+phi(2)*z(k-1)
%
n=length(batch);
for k=1:5
  batch(n+k)=phi1*batch(n+k-1)+phi2*batch(n+k-2);
end
%
% Note that these predictions are made with the demeaned series, so
% we'll still have to add the mean back in at the end.
%
% In order to put confidence intervals around our predictions, we
% need the Psi form of the model.  
%
% Psi(B)=1/(1-phi1*B-phi2*B^2)=
%  1 - 0.3198 B + 0.2820 B^2 - 0.1476 B^3 + 0.09789 B^4  - 0.05784 B^5 + ...
%
psi=[-0.3198; 0.2820; -0.1476; 0.09789]
v=zeros(5,1);
v(1)=sigmaa^2*1;
v(2)=sigmaa^2*(1+psi(1)^2);
v(3)=sigmaa^2*(1+psi(1)^2+psi(2)^2);
v(4)=sigmaa^2*(1+psi(1)^2+psi(2)^2+psi(3)^2);
v(5)=sigmaa^2*(1+psi(1)^2+psi(2)^2+psi(3)^2+psi(4)^2);
%
% We'll construct 1 standard deviation error bars for the predicted
% points.  
% 
ebar=sqrt(v);
%
% Finally, plot the data and the predicted data points, with error bars.
%
figure(2);
plot(1:n,batch(1:n)+batchmean,'ko-');
hold on
%
% The new points.  Use "*" markers and error bars.
%
errorbar((n+1):(n+5),batchmean+batch(n+1:n+5),ebar,'k*-');
%
% Connect the old and new points.
%
plot(n:n+1,batchmean+batch(n:n+1),'k-');
%
% label the axes.
%
xlabel('sample');
ylabel('output');
%
% reset the axes.
%
axis([0 80 0 100]);
print -deps batchpred.eps



